/*
    ***** BEGIN LICENSE BLOCK *****
    
    Copyright (c) 2008  Nagy Viktor
    some parts: Copyright (c) 2006 Center for History and New Media
                        George Mason University, Fairfax, Virginia, USA
                        http://chnm.gmu.edu
    
    Licensed under the Educational Community License, Version 1.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
    
    http://www.opensource.org/licenses/ecl1.php
    
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
    
    ***** END LICENSE BLOCK *****
*/

Zotero.ZoteroExport = Object
Zotero.ZoteroExport.Item = Object

Zotero.ZoteroExport = {
	selectedItem: null,
	
	init: function () {
		// Connect to (and create, if necessary) helloworld.sqlite in the Zotero directory
		
		//Register callback for showing item context menu
		var menu = document.getElementById('zotero-itemmenu');
		menu.addEventListener('popupshowing', this.addExportToFilesystemMenuItem, false);
	},
	
	export: function() {
		var items = ZoteroPane.getSelectedItems();
		Zotero.debug("zoteroexport: Reference Items selected: " + items.length);
		
		for each (var item in items) { 
			exportData = Zotero.ZoteroExport.Item.init(item)
			if (exportData.title) { // The title is always set, otherwise there was a problem
				Zotero.debug("[ZoteroExport] Suggested filename: " + exportData.title)
				this._export(exportData)
			}
		}
	},	
	
	/**
	* based on code from Zotero overlay.js ZoteroPane::showSelectedAttachmentInFilesystem
	*/
	_export: function(exportData) {
		fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker)
		fp.init(window, document.getElementById('zotero-export-strings').getString('filepicker.Title'), Components.interfaces.nsIFilePicker.modeSave)
		fp.defaultString = exportData.filename
		
		// set the start path
		prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("zoteroexport.")
     	prefs.QueryInterface(Components.interfaces.nsIPrefBranch2)
     	startPath = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
     	if (prefs.getCharPref("savePath")) {
     		startPath.initWithPath(prefs.getCharPref("savePath"))
     		fp.displayDirectory = startPath
     	}
		
		rv = fp.show()
		if (rv == Components.interfaces.nsIFilePicker.returnOK || rv == Components.interfaces.nsIFilePicker.returnReplace) {
			if (exportData.linkMode != Zotero.Attachments.LINK_MODE_LINKED_URL)
			{
				var file = exportData.file
				if (file){
					try {
						var origFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
						origFile.initWithFile(file)
						var destDir = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
						if (prefs.getBoolPref('preferences.exportType.iLiad'))
							destDir.initWithPath(fp.file.path)
						else
							destDir.initWithFile(fp.file.parent)
					} catch(e) {
						Zotero.debug("[ZoteroExport] Oops, an exception. Exiting")
						alert(document.getElementById('zotero-export-strings').getString('export.failure'))
						return false
					}
					if (!origFile || ! destDir) {
						Zotero.debug("[ZoteroExport] There was an error creating the nsILocalFiles for copying")
						alert(document.getElementById('zotero-export-strings').getString('export.failure'))
						return false
					}
					// @todo don't delete the directory, because we need the scribble file
					if (fp.file.exists()) {
						Zotero.debug("[ZoteroExport] Deleting existing file first")
						fp.file.remove(false)
					}
					Zotero.debug("[ZoteroExport] Exporting file to " + fp.file.path)
					origFile.copyTo(destDir, fp.file.leafName)
					if (prefs.getBoolPref('preferences.exportType.iLiad')) {
						manifest = this._prepareManifest(fp.file.leafName, exportData)
						this._writeDataToFile(fp.file.path, 'manifest.xml', manifest)
						scribble = this._prepareScribble(fp.file.leafName, exportData)
						this._writeDataToFile(fp.file.path, 'scribble.irx', scribble)
					}
					
					// save preferences: fp.file.parent
					prefs.setCharPref("savePath", fp.file.parent.path)
					// for simple copy: prefs.setCharPref("savePath", fp.file.leafName)
				}
			}
			alert(document.getElementById('zotero-export-strings').getString('export.success'))
		}
		Zotero.debug("[ZoteroExport] Export finished")
	},
	
	_writeDataToFile: function(destDir, filename, data) {
		file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(destDir)
		file.appendRelativePath(filename)
		if ( !file.exists() ) {
			Zotero.debug("[ZoteroExport] Creating file " + file.path)
			file.create( Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 420 );
		}
		foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                         .createInstance(Components.interfaces.nsIFileOutputStream);

		foStream.init(file, 0x02 | 0x08 | 0x20, 0666, 0); 	// write, create, truncate
		foStream.write(data, data.length);
		foStream.close();
		Zotero.debug('[ZoteroExport] Data written to ' + file.path)
	}, 
	
	_prepareManifest: function(filename, dataObj) {
		manifest = '<?xml version="1.0" encoding="utf-8"?>'
		manifest += '<package><metadata><dc-metadata>'
		// the user is likely to expect the item to appear under the name given
		manifest += '<Title>' + filename + '</Title>'
		// and we force our own description
		manifest += '<Description>' + dataObj.getDescription() + '</Description>'
		manifest += '<Date>' + dataObj.year + '</Date>'
		manifest += '<Format/><Identifier/><Language/><Type/></dc-metadata>'
		manifest += '<y-metadata>'
		manifest += '<startpage>' + filename + '</startpage>'
		manifest += '<version>000</version>'
		manifest += '<ItemSize>393216</ItemSize>'
		manifest += '</y-metadata>'
		manifest += '</metadata>'
		manifest += '<last-location><pagenumber>1</pagenumber></last-location>'
		manifest += '<viewer-settings><zoomfactor>zoomPage</zoomfactor><rotation>0</rotation>'
		manifest += '<positionx>0</positionx><positiony>0</positiony>'
		manifest += '<mode>page</mode><modefullscreen>1</modefullscreen><modetoolbar>0</modetoolbar>'
		manifest += '</viewer-settings>'
		manifest += '<pen-settings><size>3</size><color>3</color><linestyle/>'
		manifest += '<attributes>0</attributes></pen-settings>'
		manifest += '</package>'
		Zotero.debug('[ZoteroExport] ' + manifest)
		return manifest
	},
	
	_prepareScribble: function(filename) {
		return ''
	},
	
	addExportToFilesystemMenuItem: function() {
		Zotero.debug("[addExportToFilesystemMenuItem]");
		
		var menu = document.getElementById('zotero-itemmenu');
		var menuitem = null;
		
		//TODO: Properly externalize strings
		for each (var m in menu.childNodes) {
			
			if (m && m.getAttribute && m.getAttribute('label') == 'Export to the filesystem') {
				menuitem = m;
			}
		}
		
		Zotero.debug("[addExportToFilesystemMenuItem] Filesystem menu item hidden: " + menuitem.getAttribute('hidden'));
		
		//Hide the last menu item if more than 1 item selected
		if (ZoteroPane.getSelectedItems().length == 1) {
			Zotero.debug("[addExportToFilesystemMenuItem] selection  length == 1: show menu");

			menuitem.setAttribute('disabled', false);
			menuitem.setAttribute('hidden',false);
			
		}
		else {
			Zotero.debug("[addExportToFilesystemMenuItem] selection  length != 1: hide menu");
			menuitem.setAttribute('disabled', true);
			menuitem.setAttribute('hidden',true);
		}
		
	},
}

Zotero.ZoteroExport.Item = {
	item: null,
	title: null,
	path: null,
	filename: null,
	authors: null,
	year: null,
	journal: null,
	
	init: function(item) {
		this.item = item
		this.getData(item)
		return this
	},
	
	// @TODO
	listAuthors: function(creators) {
		authors = new Array()
		for each (var creator in creators) {
			authors[authors.length] = creator.lastName
		}
		Zotero.debug('[ZoteroExport] listauthors ' + authors)
		return authors.join(', ')
	},
	
	/**
	* Populate the object
	*/
	getData: function(item) {
		if (item.isAttachment()) {
			this.title = item.getField('title')
			this.file = item.getFile()
			this.filename = this.file.leafName
			this.linkMode = item.getAttachmentLinkMode()
			this.journal = item.getField('journalAbbreviation') + item.getField('bookTitle') + item.getField('shortTitle') + item.getField('publisher') + this.escapeAmpersand(item.getField('url'))
		}
		else {
			attachments = item.getAttachments()
			if (attachments.length>0) {
				first = Zotero.Items.get(attachments[0])
				/*for each (var id in attachments) {
					t = Zotero.Items.get(id)
					if (t.getAttachmentLinkMode() == Zotero.Attachments.LINK_MODE_LINKED_URL) {
						first = t
						break
					}
				}*/
				if (first) {
					Zotero.debug("[ZoteroExport] First Attachment: " + (first.getField('title')));
					this.title = item.getField('title')
					this.file = first.getFile()
					this.filename = this.file.leafName
					this.linkMode = first.getAttachmentLinkMode()
					this.year = item.getField('date')
					this.authors = this.listAuthors(item.getCreators())
					this.journal = item.getField('journalAbbreviation') + item.getField('bookTitle') + item.getField('shortTitle') + item.getField('publisher')
				}
				else {
					alert(document.getElementById('zotero-export-strings').getString('error.NotAttachmentToExport'))
				}
			}
			else {
				// what could we do if there are no attachments?
				alert(document.getElementById('zotero-export-strings').getString('error.NotFileSelected'))
			}
		}
	},
	
	escapeAmpersand: function(str) {
		return str.replace(/&/g, '&amp;')
	},
	
	/**
	* Recover the title field
	*/
	getTitle: function(){
		return this.filename
	},
	
	/**
	* Prepare the description for the manifest file
	*/
	getDescription: function() {
		desc = [this.authors, this.year, this.journal]
		desc = desc.filter( function(x) {
				return (x ? true : false)
			})
		return desc.join('; ')
	}
}

window.addEventListener('load', function(e) { Zotero.ZoteroExport.init(); }, false);
